﻿using System.Data;
using System.Data.Entity;
using System.Diagnostics;
using System.Data.Common;
using System.Data.SqlClient;
using System.Linq;

namespace CRUD
{
	class Program
	{
		static void Main(string[] args)
		{
			int id;
			byte[] version;
			Person person;
			Database.SetInitializer(new Initializer());

			//add
			using (var ctx = new Context())
			{
				ctx.Database.Initialize(false);
				person = AddNew(ctx);
				id = person.ID;
				version = person.Version;
			}

			//update
			using (var ctx = new Context())
			{
				person = Update(ctx, id, version);
				id = person.ID;
				version = person.Version;
			}

			//delete
			using (var ctx = new Context())
			{
				Delete(ctx, id, version);
			}

			//cache
			using (var ctx = new Context())
			{
                //disable tracking
				var all = ctx.People.AsNoTracking();
				Debug.Write(all.ToString());
			}

			//views select
			using (var ctx = new Context())
			{
				var people = ctx.Database.SqlQuery<ReadOnlyPerson>("SELECT * FROM PersonView").ToList();
			}

			//stored procedures select
			using (var ctx = new Context())
			{
				var parameter = new SqlParameter("@LastName", "d");
				var people = ctx.Database.SqlQuery<Person>("SearchPeople @LastName", parameter).ToList();
			}

			//stored procedures
			using (var ctx = new Context())
			{
				var parameter1 = new SqlParameter("@ID", 0) { Direction = ParameterDirection.Output, DbType = DbType.Int32 };
				var parameter2 = new SqlParameter("@FirstName", "Sergey");
				var parameter3 = new SqlParameter("@LastName", "Barskiy");
				ctx.Database.ExecuteSqlCommand("SearchPeople @LastName", parameter1, parameter2, parameter3);
				var newID = parameter1.Value;
			}

		}

		private static Person AddNew(Context ctx)
		{
			var person = new Person
			{
				FirstName = "Sergey",
				LastName = "Barskiy",
			};
			// either one is OK
			//ctx.Developers.Add(developer);
			ctx.Entry(person).State = EntityState.Added;
			ctx.SaveChanges();
			return person;
		}

		private static Person Update(Context ctx, int id, byte[] version)
		{
			var person = new Person
			{
				FirstName = "Updated",
				LastName = "Barskiy",
				Version = version,
				ID = id
			};
			// either one is OK
			//ctx.Developers.Attach(developer);
			//developer.LastName = "Something";
			ctx.Entry(person).State = EntityState.Modified;
			ctx.SaveChanges();
			return person;
		}

		private static void Delete(Context ctx, int id, byte[] version)
		{
			var developer = new Person
			{
				ID = id,
				Version = version
			};

			//bad - extra query against DB
			//ctx.People.Remove(ctx.People.Find(id));

			// either one is OK
			//ctx.Developers.Attach(developer);
			//ctx.Developers.Remove(developer);
			ctx.Entry(developer).State = EntityState.Deleted;
			ctx.SaveChanges();
			return;
		}
	}
}
