﻿using System.Configuration;
using System.Data.Entity;

namespace ComplexTypes
{
	public class Context : DbContext
	{
		public Context()
			: base(ConfigurationManager.ConnectionStrings["ComplexTypesConnString"].ConnectionString)
		{

		}

		public DbSet<Person> People { get; set; }
		public DbSet<Company> Companies { get; set; }

		protected override void OnModelCreating(DbModelBuilder modelBuilder)
		{
			base.OnModelCreating(modelBuilder);
			modelBuilder.Entity<Person>().Property(p => p.Version).IsConcurrencyToken(true).IsRowVersion();
			modelBuilder.Entity<Person>().Property(p => p.FirstName).HasMaxLength(30).IsRequired();
			modelBuilder.Entity<Person>().Property(p => p.LastName).HasMaxLength(50).IsRequired();
			modelBuilder.Entity<Person>().Ignore(p => p.NickName);

			modelBuilder.Entity<Company>().HasMany(p => p.People).WithRequired(p => p.Company).WillCascadeOnDelete();


			modelBuilder.Entity<Company>().Property(p => p.Version).IsConcurrencyToken(true).IsRowVersion();
			modelBuilder.Entity<Company>().Property(p => p.CompanyName).HasMaxLength(30).IsRequired();

			modelBuilder.Entity<Company>().Map(map =>
												   {
													   map.Properties(company =>
																	  new
																		  {
																			  company.ID,
																			  company.Address,
																			  company.CompanyName,
																			  company.Version
																		  });
													   map.ToTable("Companies");
												   });

			modelBuilder.Entity<Company>().Map(map =>
			{
				map.Properties(company => company.Logo);
				map.ToTable("CompanyBlobs");
			});

			modelBuilder.ComplexType<Address>().Property(p => p.City).HasMaxLength(50);
			modelBuilder.ComplexType<Address>().Property(p => p.State).HasMaxLength(2);
			modelBuilder.ComplexType<Address>().Property(p => p.Zip).HasMaxLength(10);
			modelBuilder.ComplexType<Address>().Property(p => p.Street).HasMaxLength(100);
		}
	}
}
