﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using VSLiveWinRTData.VSLiveService;

namespace VSLiveWinRTData
{
    public class ViewModel : BaseClass
    {
#if DEBUG
        private const string _serviceUri = @"http://localhost/WinRTWcfService/VSLiveService.svc/soap/";
#else
        private const string _serviceUri = @"http://localhost/WinRTWcfService/VSLiveService.svc/soap/";
#endif
        private VSLiveServiceClient _client;

        public ViewModel()
        {
            DeleteCommand = new SimpleCommand<Session>(OnDelete);
            SaveCommand = new SimpleCommand<object>(OnSave, CanSave);
            AddCommand = new SimpleCommand<object>(OnAdd);
            SetupClient();
            LoadData();
        }

        private ExtendedObservableCollection<Session> _sessions;

        public ExtendedObservableCollection<Session> Sessions
        {
            get { return _sessions; }
            set { _sessions = value; OnPropertyChanged("Sessions"); }
        }
        

        public async Task LoadData()
        {
            IsBusy = true;
            var sessions = await _client.GetListAsync();
            Sessions = new ExtendedObservableCollection<Session>(sessions);
            IsBusy = false;
        }

        private void SetupClient()
        {
            BasicHttpBinding binding = new BasicHttpBinding(BasicHttpSecurityMode.None);
            binding.AllowCookies = true;
            binding.MaxBufferSize = int.MaxValue;
            binding.MaxReceivedMessageSize = int.MaxValue;
            binding.OpenTimeout = TimeSpan.FromSeconds(10);
            EndpointAddress address = new EndpointAddress(_serviceUri);
            _client = new VSLiveServiceClient(binding, address);
        }

        private bool isBusy;

        public bool IsBusy
        {
            get { return isBusy; }
            set { isBusy = value; OnPropertyChanged("IsBusy"); }
        }
        

        private Session selectedSession;

        public Session SelectedSession
        {
            get { return selectedSession; }
            set
            {
                selectedSession = value;
                OnPropertyChanged("SelectedSession");
            }
        }

        public SimpleCommand<Session> DeleteCommand { get; set; }

        public async void OnDelete(Session parameter)
        {
            if (parameter != null)
            {
                if (parameter.SessionID > 0)
                {
                    await _client.DeleteAsync(parameter.SessionID);
                }
                Sessions.Remove(parameter);
            }
        }

        public SimpleCommand<object> SaveCommand { get; set; }

        public async void OnSave(object parameter)
        {
            if (_client != null)
            {
                IsBusy = true;
                Session session;
                if (selectedSession.SessionID == 0)
                {
                    session = await _client.CreateAsync(selectedSession);
                }
                else
                {
                    session = await _client.UpdateAsync(selectedSession);
                }
                Sessions[Sessions.IndexOf(selectedSession)] = session;
                SelectedSession = session;
                IsBusy = false;
            }
        }
        public bool CanSave(object parameter)
        {
            return (_client != null);
        }

        private SimpleCommand<object> addCommand;

        public SimpleCommand<object> AddCommand
        {
            get { return addCommand; }
            set { addCommand = value; }
        }

        public async void OnAdd(object parameter)
        {
            if (_client != null)
            {
                var session = new Session() { When = DateTime.Now };
                Sessions.Add(session);
                SelectedSession = session;
            }
        }
        
    }
}
