﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data.Services.Client;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Runtime.Serialization.Json;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using VSLiveWinRTData.VSLiveService;
using Windows.Storage.Streams;
using Windows.UI.Core;
using Windows.UI.Xaml;

namespace VSLiveWinRTData
{
    public class WcfDataServicesViewModel : BaseClass
    {
#if DEBUG
        private const string _serviceUri = @"http://localhost/WinRTWcfService/VSLiveWcfDataService.svc/";
#else
        private const string _serviceUri = @"http://localhost/WinRTWcfService/VSLiveWcfDataService.svc/";
#endif

        private readonly WinRT.OData.Context _oDataClient;
        CoreDispatcher _dispatcher;

        public WcfDataServicesViewModel()
        {
            DeleteCommand = new SimpleCommand<WinRT.OData.Session>(OnDelete);
            SaveCommand = new SimpleCommand<object>(OnSave, CanSave);
            AddCommand = new SimpleCommand<object>(OnAdd);
            _oDataClient = new WinRT.OData.Context(new Uri(_serviceUri));
        }

        public DataServiceCollection<WinRT.OData.Session> Items { get; private set; }

        private ExtendedObservableCollection<WinRT.OData.Session> _sessions;

        public ExtendedObservableCollection<WinRT.OData.Session> Sessions
        {
            get { return _sessions; }
            set { _sessions = value; OnPropertyChanged("Sessions"); }
        }


        public void LoadData(CoreDispatcher dispatcher)
        {
            try
            {
                _dispatcher = dispatcher;
                _oDataClient.BeginExecute<WinRT.OData.Session>(new Uri(string.Concat(_serviceUri, "Sessions")),
                    (ar) =>
                    {
                        _dispatcher.InvokeAsync(
                            Windows.UI.Core.CoreDispatcherPriority.Normal,
                            (s, a) =>
                            {
                                var ctx = ar.AsyncState as WinRT.OData.Context;
                                var data = ctx.EndExecute<WinRT.OData.Session>(ar);
                                Items = new DataServiceCollection<WinRT.OData.Session>(data, TrackingMode.AutoChangeTracking);
                                Sessions = new ExtendedObservableCollection<WinRT.OData.Session>(Items);
                            }, this, null);

                    }, _oDataClient);

            }
            catch (DataServiceClientException ex)
            {
                // handle error message
            }
        }


        private bool isBusy;

        public bool IsBusy
        {
            get { return isBusy; }
            set { isBusy = value; OnPropertyChanged("IsBusy"); }
        }


        private WinRT.OData.Session selectedSession;

        public WinRT.OData.Session SelectedSession
        {
            get { return selectedSession; }
            set
            {
                selectedSession = value;
                OnPropertyChanged("SelectedSession");
            }
        }

        public SimpleCommand<WinRT.OData.Session> DeleteCommand { get; set; }

        public async void OnDelete(WinRT.OData.Session parameter)
        {
            if (parameter != null)
            {
                if (parameter.SessionID > 0)
                {
                    IsBusy = true;

                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
                else
                {
                    Sessions.Remove(parameter);
                    IsBusy = false;
                }
            }
        }

        public SimpleCommand<object> SaveCommand { get; set; }

        public async void OnSave(object parameter)
        {
            _oDataClient.BeginSaveChanges(
             SaveChangesOptions.Batch,
             result =>
               _dispatcher.InvokeAsync(
                    Windows.UI.Core.CoreDispatcherPriority.Normal,
                   (s, e) =>
                   {

                       if (result.IsCompleted)
                       {
                           var client = result.AsyncState as WinRT.OData.Context;
                           Debug.Assert(client != null, "client != null");
                           DataServiceResponse response = client.EndSaveChanges(result);
                           if (response.IsBatchResponse)
                           {
                               var errors = new StringBuilder();
                               foreach (ChangeOperationResponse change in response)
                               {
                                   if (change.Error != null)
                                   {
                                       errors.Append(change.Error.Message);
                                   }
                                   else
                                   {
                                       if (change.Descriptor is EntityDescriptor)
                                       {
                                           var descriptor = (EntityDescriptor)change.Descriptor;
                                           var session = (WinRT.OData.Session)descriptor.Entity;
                                           if (descriptor.State != EntityStates.Detached)
                                           {
                                               if (!Items.Any(one => one.SessionID == session.SessionID))
                                               {
                                                   Items.Add(session);
                                               }
                                           }
                                       }
                                   }
                               }
                               if (errors.Length != 0)
                               {
                                   // handle error
                               }
                           }
                       }
                   }, _oDataClient, null),
             _oDataClient);
        }

        public bool CanSave(object parameter)
        {
            return (_oDataClient != null);
        }

        private SimpleCommand<object> addCommand;

        public SimpleCommand<object> AddCommand
        {
            get { return addCommand; }
            set { addCommand = value; }
        }

        public async void OnAdd(object parameter)
        {
            if (_oDataClient != null)
            {
                var session = new WinRT.OData.Session() { When = DateTime.Now };
                Sessions.Add(session);
                SelectedSession = session;
            }
        }

    }
}
